/**
 * 验证码插件
 */
const Redis = require('./redis');
const Log = require('./log');
const _ = require('./lodash');

const captchaPrefix = 'captcha_';
const expiredTime = 10 * 60; // ten minutes

module.exports = {
  /**
   * 生成验证码
   * @param mobile
   * @param opts
   *   - {Integer} expired default 600
   * @returns {Promise.<*>}
   */
  async generate(mobile, opts) {
    opts = opts || {};

    const expired = opts.expired || expiredTime;

    // get value from redis
    let value = await Redis.get(captchaPrefix + mobile);
    if (!value) {
      // generate new captcha
      let captcha = '';
      for (let i = 0; i < 4; i++) {
        const num = _.random(9);
        captcha += `${num}`;
      }

      // save to redis
      Redis.set(captchaPrefix + mobile, captcha, expired);
      value = captcha;
    }

    Log.info(`captcha ${mobile} ${value}`);
    return value;
  },

  /**
   * 校验验证码
   * @param captcha
   * @param mobile
   * @returns {Promise.<boolean>}
   */
  async check(captcha, mobile) {
    const value = await Redis.get(captchaPrefix + mobile);
    if (!value || value !== captcha) {
      Log.error(`captcha incorrect, captcha ${captcha} ${mobile}`);
      return false;
    }

    return true;
  },
};
