/**
 * excel组件
 */
const exceljs = require('exceljs');

const _ = require('./lodash');

module.exports = {
  /**
   * simple write xlsx
   * @param fileName
   * @param columns
   * @param rows
   * @param opts
   *   - {String} sheetName default 'Sheet1'
   * @returns {Promise.<void>}
   */
  async simpleWrite(fileName, columns, rows, opts) {
    opts = opts || {};

    const sheetName = opts.sheetName || 'Sheet1';

    const workbook = new exceljs.Workbook();
    const worksheet = workbook.addWorksheet(sheetName);

    worksheet.columns = columns;

    _.forEach(rows, (row) => {
      worksheet.addRow(row);
    });

    await workbook.xlsx.writeFile(fileName);
  },

  /**
   * simple read xlsx
   * @param filePath
   * @param column
   * @param opts
   *   - {String} sheetName default 'Sheet1'
   * @returns {Promise.<*>}
   */
  async simpleRead(filePath, column, opts) {
    opts = opts || {};

    const sheetName = opts.sheetName || 'Sheet1';

    const values = [];
    const workbook = new exceljs.Workbook();

    // 乐启的
    const wb1 = await workbook.xlsx.readFile(filePath);
    const ws1 = wb1.getWorksheet(sheetName);
    const col = ws1.getColumn(column);

    col.eachCell((cell) => {
      const value = cell.value;
      values.push(value);
    });

    return values;
  },
};
