/**
 * 行为认证组件
 */
const Geetest = require('gt3-sdk');

const Log = require('./log');
const CoreError = require('./errors/core');

let res = {
  id: 'd1176b56be795f2e752243f68e5290d8', // 申请的 id
  key: '6e1b9347f219da19e1ffc9eabf8bf801', // 申请的 key
};

try {
  res = require(`${process.cwd()}/configs/${process.env.NODE_ENV}/gt.json`);
} catch (e) {
  Log.warn(`Not found gt.json from configs/${process.env.NODE_ENV} folder. Use default value ${JSON.stringify(res)}`);
}

const geetest = new Geetest({
  geetest_id: res.id,
  geetest_key: res.key,
});

module.exports = {
  /**
   * 获取页面调用所需参数
   * @return {Promise<*>}
   */
  async register() {
    const data = await geetest.register(null);

    return data;
  },


  /**
   * 二次验证
   * @param challenge
   * @param validate
   * @param seccode
   * @return {Promise<void>}
   */
  async vertify(challenge, validate, seccode) {
    // 向极验申请每次验证所需的challenge
    const data = await geetest.validate(false, {
      geetest_challenge: challenge,
      geetest_validate: validate,
      geetest_seccode: seccode,
    });

    if (!data) {
      throw new CoreError('二次验证失败');
    }
  },
};
