/**
 * 微信支付组件
 */
const tenpay = require('tenpay');
const fs = require('fs');

const Log = require('../log');

let res = {
  // 公众号id
  appid: 'wx5207a3ed058b5280',
  // 公众号密钥
  appSecret: '2ab80446e6fb3c6a3858f4e805cb4817',
  // 商户号id
  mchid: '1510196431',
  // 商户号密钥
  partnerKey: 'f07daf94c7244f07bc3e7486d084ed74',
  // 证书文件路径
  pfx: 'apiclient_cert.p12',
  // 支付回调网址
  notify_url: '',
  // 退款回调网址
  refund_url: '',
  // ip地址
  spbill_create_ip: '',
};

try {
  res = require(`${process.cwd()}/configs/${process.env.NODE_ENV}/wechat.json`);
  const pfx = fs.readFileSync(`${process.cwd()}/configs/${process.env.NODE_ENV}/${res.pfx}`);
  res.pfx = pfx;
} catch (e) {
  Log.warn(`Not found wechat.json from configs/${process.env.NODE_ENV} folder. Use default value ${JSON.stringify(res)}`);
}

const api = tenpay.init(res);

module.exports = {
  api,

  /**
   * 获取微信JSSDK支付参数(自动下单, 兼容小程序)
   * @param outTradeNo
   * @param body
   * @param totalFee
   * @param openId
   * @param opts
   *    - {String} attach default ''
   *    - {String} notifyUrlCode default ''
   * @returns {Promise<*>}
   */
  async getPayParams(outTradeNo, body, totalFee, openId, opts) {
    opts = opts || {};

    const attach = opts.attach || '';
    const notifyUrlCode = opts.notifyUrlCode || '';

    const notifyUrl = notifyUrlCode ? res[notifyUrlCode] : '';

    const result = await api.getPayParams({
      // 商户内部订单号
      out_trade_no: outTradeNo,
      // 商品简单描述
      body,
      // 订单金额(分)
      total_fee: totalFee,
      // 用户openid
      openid: openId,
      // 附加参数
      attach,
      // 回调地址
      notify_url: notifyUrl,
    });

    return result;
  },

  /**
   * 获取APP支付参数(自动下单)
   * @param outTradeNo
   * @param body
   * @param totalFee
   * @param opts
   *    - {String} attach default ''
   *    - {String} notifyUrlCode default ''
   * @returns {Promise<*>}
   */
  async getAppParams(outTradeNo, body, totalFee, opts) {
    opts = opts || {};

    const attach = opts.attach || '';
    const notifyUrlCode = opts.notifyUrlCode || '';

    const notifyUrl = notifyUrlCode ? res[notifyUrlCode] : '';

    const result = await api.getAppParams({
      // 商户内部订单号
      out_trade_no: outTradeNo,
      // 商品简单描述
      body,
      // 订单金额(分)
      total_fee: totalFee,
      // 附加参数
      attach,
      // 回调地址
      notify_url: notifyUrl,
    });

    return result;
  },

  /**
   * 微信统一下单
   * @param outTradeNo
   * @param body
   * @param totalFee
   * @param openId
   * @param opts
   *    - {String} attach default ''
   *    - {String} notifyUrlCode default ''
   * @returns {Promise<void>}
   */
  async unifiedOrder(outTradeNo, body, totalFee, openId, opts) {
    opts = opts || {};

    const attach = opts.attach || '';
    const notifyUrlCode = opts.notifyUrlCode || '';

    const notifyUrl = notifyUrlCode ? res[notifyUrlCode] : '';

    const result = await api.unifiedOrder({
      // 商户内部订单号
      out_trade_no: outTradeNo,
      // 商品简单描述
      body,
      // 订单金额(分)
      total_fee: totalFee,
      // 用户openid
      openid: openId,
      // 附加参数
      attach,
      // 回调地址
      notify_url: notifyUrl,
    });

    return result;
  },

  /**
   * 申请退款
   * @param outTradeNo
   * @param outRefundNo
   * @param totalFee
   * @param refundFee
   * @param opts
   *    - {String} notifyUrlCode default ''
   * @returns {Promise<void>}
   */
  async refundOrder(outTradeNo, outRefundNo, totalFee, refundFee, opts) {
    opts = opts || {};

    const notifyUrlCode = opts.notifyUrlCode || '';

    const notifyUrl = notifyUrlCode ? res[notifyUrlCode] : '';

    const result = await api.refund({
      // 商户内部订单号
      out_trade_no: outTradeNo,
      // 商户内部退款单号
      out_refund_no: outRefundNo,
      // 订单金额(分)
      total_fee: totalFee,
      // 退款金额(分)
      refund_fee: refundFee,
      // 回调地址
      notify_url: notifyUrl,
    });

    return result;
  },

  /**
   * 企业付款
   * @param partnerTradeNo
   * @param openId
   * @param amount
   * @param desc
   * @param opts
   *    - {String} checkName default 'FORCE_CHECK'
   * @returns {Promise<void>}
   */
  async transfers(partnerTradeNo, openId, amount, desc, opts) {
    opts = opts || {};

    const checkName = opts.checkName || 'FORCE_CHECK';

    const result = await api.transfers({
      // 商户内部付款订单号
      partner_trade_no: partnerTradeNo,
      // openId
      openid: openId,
      // 付款金额(分)
      amount,
      // 企业付款描述信息
      desc,
      // 是否验证真实姓名
      check_name: checkName,
    });

    return result;
  },
};
