/**
 * redis组件
 */
const Redis = require('ioredis');

const Log = require('./log');

let res = {
  host: 'localhost',
  port: 6379,
};

try {
  res = require(`${process.cwd()}/configs/${process.env.NODE_ENV}/redis.json`);
} catch (e) {
  Log.warn(`Not found redis.json from configs/${process.env.NODE_ENV} folder. Use default value ${JSON.stringify(res)}`);
}

const redis = new Redis(res.port, res.host);

module.exports = {
  redis, // redis instance

  /**
   * set
   * @param key
   * @param value
   * @param time (unit: second)
   */
  set(key, value, time) {
    if (arguments.length === 2) {
      redis.set(key, value);
      return;
    }

    redis.set(key, value, 'EX', time);
  },

  /**
   * get
   * @param key
   * @returns {Promise.<*>}
   */
  async get(key) {
    const result = await redis.get(key);
    return result;
  },
};
