/**
 * token组件
 */
const jsonwebtoken = require('jsonwebtoken');

const TokenError = require('./errors/token');

let secret;
let expired;

module.exports = {
  /**
   * init
   * @param tokenSecret
   * @param tokenExpired
   */
  init(tokenSecret, tokenExpired) {
    secret = tokenSecret;
    expired = tokenExpired;
  },

  /**
   * set
   * @param token
   * @returns {*}
   */
  set(token) {
    const tokenValue = jsonwebtoken.sign({
      token: token || {},
    }, secret, { expiresIn: expired });

    return tokenValue;
  },

  /**
   * get
   * @param tokenValue
   * @returns {*}
   */
  get(tokenValue) {
    try {
      const decodeToken = jsonwebtoken.verify(tokenValue, secret);
      return decodeToken.token;
    } catch (e) {
      throw new TokenError();
    }
  },
};
