/**
 * 工具组件
 */
const Model = require('./model');
const Moment = require('./moment');
const Validator = require('./validator');
const Pinyin = require('./pinyin');

module.exports = {
  /**
   * 事务
   * @param transaction
   * @returns {Promise.<void>}
   */
  async transaction(transaction) {
    await Model.sequelize.transaction(transaction).catch((err) => {
      throw new Error(err.message);
    });
  },

  dateTimeFormat(value) {
    const formatter = 'YYYY-MM-DD HH:mm:ss';

    if (value) {
      return Moment(value).format(formatter);
    }

    return Moment().format(formatter);
  },

  dateFormat(value) {
    const formatter = 'YYYY-MM-DD';

    if (value) {
      return Moment(value).format(formatter);
    }

    return Moment().format(formatter);
  },

  timeFormat(value) {
    const formatter = 'HH:mm:ss';

    if (value) {
      return Moment(value).format(formatter);
    }

    return Moment().format(formatter);
  },

  addDays(opts) {
    opts = opts || {};

    const value = opts.value || '';
    const days = opts.days || 1;

    if (value) {
      return Moment(value).add(days, 'days');
    }

    return Moment().add(days, 'days');
  },

  minusDays(opts) {
    opts = opts || {};

    const value = opts.value || '';
    const days = opts.days || 1;

    if (value) {
      return Moment(value).subtract(days, 'days');
    }

    return Moment().subtract(days, 'days');
  },

  isMobile(mobile) {
    return Validator.isMobilePhone(mobile, 'zh-CN');
  },

  // 汉字转拼音首字母缩写
  coverToFirstLetter(value) {
    const result = Pinyin(value, {
      style: Pinyin.STYLE_FIRST_LETTER,
    });

    return result.join('');
  },

  // 正常查询/分页查询
  async findAllOrPage(model, pagination, opts) {
    pagination = pagination || {};
    opts = opts || {};

    const page = pagination.page || false;

    // 分页查询
    if (page) {
      let current = Number(pagination.current || 1);
      let size = Number(pagination.size || 10);

      if (current < 1) {
        current = 1;
      }

      if (size < 1) {
        size = 1;
      }

      opts.offset = (current - 1) * size;
      opts.limit = size;

      const total = await model.count(opts);

      const pageRecords = await model.findAndCountAll(opts);

      return {
        total,
        records: pageRecords.rows,
        current,
        size,
      };
    }

    // 正常查询
    const records = await model.findAll(opts);

    return records;
  },
};
