/**
 * 微信组件
 */
const WechatAPI = require('co-wechat-api');
const WechatOAuth = require('co-wechat-oauth');

const Log = require('./log');
const Redis = require('./redis');

let res = {
  // 公众号id
  appid: 'wx5207a3ed058b5280',
  // 公众号密钥
  appSecret: '2ab80446e6fb3c6a3858f4e805cb4817',
};

try {
  res = require(`${process.cwd()}/configs/${process.env.NODE_ENV}/wechat.json`);
} catch (e) {
  Log.warn(`Not found wechat.json from configs/${process.env.NODE_ENV} folder. Use default value ${JSON.stringify(res)}`);
}

const api = new WechatAPI(res.appid, res.appSecret, async () => {
  const token = await Redis.get('api_access_token');
  return token;
}, (token) => {
  Redis.set('api_access_token', token);
});

const oauthApi = new WechatOAuth(res.appid, res.appSecret, async (openid) => {
  const token = await Redis.get(`${openid}_oauth_access_token`);
  return token;
}, async (openid, token) => {
  Redis.set(`${openid}_oauth_access_token`, token);
});

module.exports = {
  /**
   * 生成重定向地址
   * @param redirectUrl
   * @param state
   * @param scope
   * @returns {*}
   */
  generateRedirectUrl(redirectUrl, state, scope) {
    const url = oauthApi.getAuthorizeURL(redirectUrl, state, scope);
    return url;
  },

  /**
   * 生成重定向地址(PC)
   * @param redirectUrl
   * @param state
   * @param scope
   * @returns {*}
   */
  generateRedirectUrlForWebsite(redirectUrl, state, scope) {
    const url = oauthApi.getAuthorizeURLForWebsite(redirectUrl, state, scope);
    return url;
  },

  /**
   * 获取AccessToken和openId
   * @param code
   * @returns {Promise<{accessToken: *, openId: (*|String|Object)}>}
   */
  async getAccessTokenAndOpenId(code) {
    const token = await oauthApi.getAccessToken(code);

    return {
      accessToken: token.data.access_token,
      openId: token.data.openid,
    };
  },

  /**
   * 获取js参数
   * @param jsApiList
   * @param url
   * @param debug
   * @returns {Promise<*>}
   */
  async getJsConfig(jsApiList, url, debug) {
    const param = {
      debug: debug || false,
      jsApiList,
      url,
    };

    const result = await api.getJsConfig(param);
    return result;
  },
};
