/**
 * 渲染中间件
 */
const Log = require('../components/log');
const CoreError = require('../components/errors/core');
const ValidateError = require('../components/errors/validate');
const SecurityError = require('../components/errors/security');
const TokenError = require('../components/errors/token');

class ApiRender {
  constructor(ctx, logEnable, tokenErrorCode) {
    this.ctx = ctx;
    this.logEnable = logEnable;
    this.tokenErrorCode = tokenErrorCode;
  }

  success(data) {
    this.ctx.response.status = 200;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'success',
      message: '请求成功',
      data: data || '',
    };

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} ${JSON.stringify(this.ctx.response.body)}`);
    }
  }

  tokenError(e) {
    this.ctx.response.status = 401;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: this.tokenErrorCode,
      message: '认证异常',
      data: '',
    };

    Log.error(e);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }

  coreError(e) {
    this.ctx.response.status = 500;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'core_error',
      message: e.message,
      data: '',
    };

    Log.error(e);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }

  validateError(e) {
    this.ctx.response.status = 400;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'validate_error',
      message: '参数验证异常',
      data: '',
    };

    Log.error(e);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }

  securityError(e) {
    this.ctx.response.status = 403;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'security_error',
      message: '安全验证异常',
      data: '',
    };

    Log.error(e);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }

  systemError(e) {
    this.ctx.response.status = 500;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'system_error',
      message: '系统异常',
      data: '',
    };

    Log.error(e);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }

  notFoundError() {
    this.ctx.response.status = 404;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'not_found_error',
      message: '路由不存在',
      data: '',
    };

    Log.error(`not found the route ${this.ctx.request.url} `);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }

  rateLimitError() {
    this.ctx.response.status = 429;
    this.ctx.response.type = 'application/json';
    this.ctx.response.body = {
      code: 'rate_limit_error',
      message: '限流异常',
      data: '',
    };

    Log.error(`rate limit the route ${this.ctx.request.url} `);

    if (this.logEnable) {
      Log.info(`render finish ${this.ctx.request.url} `);
    }
  }
}

module.exports = (logEnable, tokenErrorCode) => async (ctx, next) => {
  ctx.render = new ApiRender(ctx, logEnable, tokenErrorCode);

  try {
    await next();
  } catch (e) {
    if (e.status === 401) {
      ctx.render.tokenError(e);
    } else if (e.status === 429) {
      ctx.render.rateLimitError();
    } else if (e instanceof CoreError) {
      ctx.render.coreError(e);
    } else if (e instanceof ValidateError) {
      ctx.render.validateError(e);
    } else if (e instanceof SecurityError) {
      ctx.render.securityError(e);
    } else if (e instanceof TokenError) {
      ctx.render.tokenError(e);
    } else {
      ctx.render.systemError(e);
    }
  }
};
